@@ -1,9 +1,11 @@
 import io
 from numbers import Number
-from typing import Any, Callable, Dict, Generator, NamedTuple, Optional, Sequence, Union
+from typing import Any, Callable, Dict, Optional, Sequence, Union
 import numpy as np
 import orjson
 import h5py
+import tifffile
+
 from .utils import sanitize_array
 
 
@@ -17,7 +19,7 @@ def bin_encode(array: Sequence[Number]) -> bytes:
     return sanitized_array.tobytes()
 
 
-def orjson_default(o) -> Union[list, str, None]:
+def orjson_default(o: Any) -> Union[list, str, None]:
     """Converts Python objects to JSON-serializable objects.
 
     :raises TypeError: if the object is not supported."""
@@ -46,43 +48,46 @@ def orjson_encode(content: Any, default: Optional[Callable] = None) -> bytes:
     return orjson.dumps(content, default=default, option=orjson.OPT_SERIALIZE_NUMPY)
 
 
-def npy_stream(array: Sequence[Number]) -> Generator[bytes, None, None]:
-    """Generator to stream nD array as a .npy file.
+def csv_encode(data: np.ndarray) -> bytes:
+    """Encodes a NumPy array in CSV.
 
-    :param array: Data to stream
+    :param: data: NumPy array to encode
     """
-    sanitized_array = sanitize_array(array)
-
-    # Stream header
     with io.BytesIO() as buffer:
-        np.lib.format.write_array_header_1_0(
-            buffer, np.lib.format.header_data_from_array_1_0(sanitized_array)
-        )
-        header = buffer.getvalue()
-    yield header
-
-    # Taken from numpy.lib.format.write_array
-    if sanitized_array.itemsize == 0:
-        buffersize = 0
-    else:
-        # Set buffer size to 16 MiB to hide the Python loop overhead.
-        buffersize = max(16 * 1024 ** 2 // sanitized_array.itemsize, 1)
-
-    for chunk in np.nditer(
-        sanitized_array,
-        flags=["external_loop", "buffered", "zerosize_ok"],
-        buffersize=buffersize,
-        order="C",
-    ):
-        yield chunk.tobytes("C")
+        np.savetxt(buffer, data, delimiter=",")
+        return buffer.getvalue()
 
 
-class Response(NamedTuple):
-    content: Union[Generator[bytes, None, None], bytes]
-    """ Encoded `content` as a generator of bytes """
+def npy_encode(data: np.ndarray) -> bytes:
+    """Encodes a NumPy array in NPY.
+
+    :param: data: NumPy array to encode
+    """
+    with io.BytesIO() as buffer:
+        np.save(buffer, data)
+        return buffer.getvalue()
+
+
+def tiff_encode(data: np.ndarray) -> bytes:
+    """Encodes a NumPy array in TIFF. The data should be 2D.
+
+    :param: data: NumPy array to encode
+    """
+    with io.BytesIO() as buffer:
+        tifffile.imwrite(buffer, data, photometric="minisblack")
+        return buffer.getvalue()
+
+
+class Response:
+    content: bytes
+    """ Encoded `content` as bytes """
     headers: Dict[str, str]
     """ Associated headers """
 
+    def __init__(self, content: bytes, headers: Dict[str, str]):
+        self.content = content
+        self.headers = {**headers, "Content-Length": str(len(content))}
+
 
 def encode(content, encoding: Optional[str] = "json") -> Response:
     """Encode content in given encoding.
@@ -92,9 +97,12 @@ def encode(content, encoding: Optional[str] = "json") -> Response:
     :param content: Content to encode
     :param encoding:
         - `json` (default)
-        - `npy`: Only nD array-like of numbers is supported
+        - `bin`: nD array/scalars in bytes
+        - `csv`: nD arrays in downloadable csv files
+        - `npy`: nD arrays in downloadable npy files
+        - `tiff`: 2D arrays in downloadable TIFF files
     :returns: A Response object containing content and headers
-    :raises ValueError: If encoding is not "json" nor "npy"
+    :raises ValueError: If encoding is not among the ones above.
     """
     if encoding in ("json", None):
         return Response(
@@ -102,15 +110,6 @@ def encode(content, encoding: Optional[str] = "json") -> Response:
             headers={"Content-Type": "application/json"},
         )
 
-    if encoding == "npy":
-        return Response(
-            npy_stream(content),
-            headers={
-                "Content-Type": "application/octet-stream",
-                "Content-Disposition": 'attachment; filename="data.npy"',
-            },
-        )
-
     if encoding == "bin":
         return Response(
             bin_encode(content),
@@ -119,4 +118,31 @@ def encode(content, encoding: Optional[str] = "json") -> Response:
             },
         )
 
+    if encoding == "csv":
+        return Response(
+            csv_encode(content),
+            headers={
+                "Content-Type": "text/csv",
+                "Content-Disposition": 'attachment; filename="data.csv"',
+            },
+        )
+
+    if encoding == "npy":
+        return Response(
+            npy_encode(content),
+            headers={
+                "Content-Type": "application/octet-stream",
+                "Content-Disposition": 'attachment; filename="data.npy"',
+            },
+        )
+
+    if encoding == "tiff":
+        return Response(
+            tiff_encode(content),
+            headers={
+                "Content-Type": "image/tiff",
+                "Content-Disposition": 'attachment; filename="data.tiff"',
+            },
+        )
+
     raise ValueError(f"Unsupported encoding {encoding}")
