@@ -1,12 +1,13 @@
 """Helpers for usage with `Tornado <https://www.tornadoweb.org>`_"""
 import os
-from typing import Any, Generator, Optional
+from typing import Any, Optional
 import h5py
 from tornado.web import RequestHandler, MissingArgumentError, HTTPError
 
 from .content import DatasetContent, ResolvedEntityContent, create_content
 from .encoders import encode
-from .utils import NotFoundError, parse_bool_arg
+from .models import LinkResolution
+from .utils import NotFoundError, parse_bool_arg, parse_link_resolution_arg
 
 
 __all__ = [
@@ -22,7 +23,7 @@ __all__ = [
 class BaseHandler(RequestHandler):
     """Base class for h5grove handlers"""
 
-    def initialize(self, base_dir, allow_origin=None) -> None:
+    def initialize(self, base_dir: str, allow_origin: Optional[str] = None) -> None:
         self.base_dir = base_dir
         self.allow_origin = allow_origin
 
@@ -44,19 +45,15 @@ class BaseHandler(RequestHandler):
             except NotFoundError as e:
                 raise HTTPError(status_code=404, reason=str(e))
 
-        chunks, headers = encode(content, format_arg)
+        response = encode(content, format_arg)
 
-        for key, value in headers.items():
+        for key, value in response.headers.items():
             self.set_header(key, value)
 
-        if isinstance(chunks, Generator):
-            for chunk in chunks:
-                self.write(chunk)
-        else:
-            self.write(chunks)
+        self.write(response.content)
         self.finish()
 
-    def get_content(self, h5file, path):
+    def get_content(self, h5file: h5py.File, path: Optional[str]):
         raise NotImplementedError
 
     def prepare(self):
@@ -71,7 +68,7 @@ class BaseHandler(RequestHandler):
 class AttributeHandler(BaseHandler):
     """/attr/ endpoint handler"""
 
-    def get_content(self, h5file, path):
+    def get_content(self, h5file: h5py.File, path: Optional[str]):
         content = create_content(h5file, path)
         assert isinstance(content, ResolvedEntityContent)
 
@@ -83,7 +80,7 @@ class AttributeHandler(BaseHandler):
 class DataHandler(BaseHandler):
     """/data/ endpoint handler"""
 
-    def get_content(self, h5file, path):
+    def get_content(self, h5file: h5py.File, path: Optional[str]):
         selection = self.get_query_argument("selection", None)
         flatten = parse_bool_arg(
             self.get_query_argument("flatten", None), fallback=False
@@ -97,9 +94,10 @@ class DataHandler(BaseHandler):
 class MetadataHandler(BaseHandler):
     """/meta/ endpoint handler"""
 
-    def get_content(self, h5file, path):
-        resolve_links = parse_bool_arg(
-            self.get_query_argument("resolve_links", None), fallback=True
+    def get_content(self, h5file: h5py.File, path: Optional[str]):
+        resolve_links = parse_link_resolution_arg(
+            self.get_query_argument("resolve_links", None),
+            fallback=LinkResolution.ONLY_VALID,
         )
         content = create_content(h5file, path, resolve_links)
         return content.metadata()
@@ -108,7 +106,7 @@ class MetadataHandler(BaseHandler):
 class StatisticsHandler(BaseHandler):
     """/stats/ endpoint handler"""
 
-    def get_content(self, h5file, path):
+    def get_content(self, h5file: h5py.File, path: Optional[str]):
         selection = self.get_query_argument("selection", None)
 
         content = create_content(h5file, path)
